/*
 * Copyright (c) 2008 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup net
 * @{
 */

/** @file
 */
#include <async.h>
#include <errno.h>
#include <ipc/ipc.h>
#include <ipc/services.h>

#include <stdio.h>

#include "modules.h"

#define MODULE_WAIT_TIME	10000

int connect_to_service( services_t need ){
	int phone;

	phone = ipc_connect_me_to( PHONE_NS, need, 0, 0 );
	if( phone < 0 ){
		do{
			usleep( MODULE_WAIT_TIME );
			phone = ipc_connect_me_to( PHONE_NS, need, 0, 0 );
		}while( phone < 0 );
	}
	return phone;
}

int start_service( services_t me, services_t need[], int * need_phone[], void ( * client_connection )( ipc_callid_t iid, ipc_call_t * icall ), int ( * initialize_me )( void )){
	ipcarg_t phonehash;
	ERROR_DECLARE;

	if(( ! me ) || ( ! client_connection )) return EINVAL;
	async_set_client_connection( client_connection );

	if( need ){
		if( ! need_phone ) return EINVAL;
		while( * need ){
			if( ! * need_phone ) return EINVAL;

			/* Connect to the needed service */
			** need_phone = connect_to_service( * need );
			++ need;
			++ need_phone;
		}
	}

	/* Initialize service by its callback */
	if( initialize_me ) ERROR_PROPAGATE( initialize_me());

	/* Register service at NS */
	ERROR_PROPAGATE( REGISTER_ME( me, & phonehash ));

	async_manager();

	return EOK;
}

/** @}
 */
